@@ -24,7 +24,7 @@ WWW::YoutubeViewer - A very easy interface to YouTube.
 
 =cut
 
-our $VERSION = '3.7.5';
+our $VERSION = '3.7.6';
 
 =head1 SYNOPSIS
 
@@ -84,8 +84,6 @@ my %valid_options = (
     prefer_mp4    => {valid => [1, 0], default => 0},
     prefer_av1    => {valid => [1, 0], default => 0},
 
-    use_invidious_api => {valid => [1, 0], default => 0},
-
     # API/OAuth
     key           => {valid => [qr/^.{15}/], default => undef},
     client_id     => {valid => [qr/^.{15}/], default => undef},
@@ -248,8 +246,7 @@ sub set_lwp_useragent {
                my ($response) = @_;
                my $code = $response->code;
 
-               $code >= 500                                # do not cache any bad response
-                 or $code == 401                           # don't cache an unauthorized response
+               $code >= 300                                # do not cache any bad response
                  or $response->request->method ne 'GET'    # cache only GET requests
 
                  # don't cache if "cache-control" specifies "max-age=0" or "no-store"
@@ -523,10 +520,14 @@ sub _extract_from_invidious {
     return @formats;
 }
 
+sub _ytdl_is_available {
+    (state $x = system('youtube-dl', '--version')) == 0;
+}
+
 sub _extract_from_ytdl {
     my ($self, $videoID) = @_;
 
-    ((state $x = system('youtube-dl', '--version')) == 0) || return;
+    $self->_ytdl_is_available() || return;
 
     my $json = $self->proxy_stdout('youtube-dl', '--all-formats', '--dump-single-json',
                                    quotemeta("https://www.youtube.com/watch?v=" . $videoID));
@@ -557,30 +558,33 @@ sub _fallback_extract_urls {
 
     my @formats;
 
-    if ($self->get_use_invidious_api) {    # use the API of invidio.us
+    # Use youtube-dl
+    if ($self->_ytdl_is_available) {
 
         if ($self->get_debug) {
-            say STDERR ":: Using invidio.us to extract the streaming URLs...";
+            say STDERR ":: Using youtube-dl to extract the streaming URLs...";
         }
 
-        push @formats, $self->_extract_from_invidious($videoID);
+        push @formats, $self->_extract_from_ytdl($videoID);
 
         if ($self->get_debug) {
-            say STDERR ":: Found ", scalar(@formats), " streaming URLs.";
+            my $count = scalar(@formats);
+            say STDERR ":: youtube-dl: found $count streaming URLs...";
         }
 
         @formats && return @formats;
     }
 
+    # Use the API of invidio.us
     if ($self->get_debug) {
-        say STDERR ":: Using youtube-dl to extract the streaming URLs...";
+        say STDERR ":: Using invidio.us to extract the streaming URLs...";
     }
 
-    push @formats, $self->_extract_from_ytdl($videoID);
+    push @formats, $self->_extract_from_invidious($videoID);
 
     if ($self->get_debug) {
         my $count = scalar(@formats);
-        say STDERR ":: Found $count streaming URLs...";
+        say STDERR ":: invidious: found $count streaming URLs...";
     }
 
     return @formats;
@@ -648,6 +652,43 @@ sub _group_keys_with_values {
     return @hashes;
 }
 
+sub _check_streaming_urls {
+    my ($self, $videoID, $results) = @_;
+
+    foreach my $video (@$results) {
+
+        if (   exists $video->{s}
+            or exists $video->{signatureCipher}
+            or exists $video->{cipher}) {    # has an encrypted signature :(
+
+            if ($self->get_debug) {
+                say STDERR ":: Detected an encrypted signature...";
+            }
+
+            my @formats = $self->_fallback_extract_urls($videoID);
+
+            foreach my $format (@formats) {
+                foreach my $ref (@$results) {
+                    if (defined($ref->{itag}) and ($ref->{itag} eq $format->{itag})) {
+                        $ref->{url} = $format->{url};
+                        last;
+                    }
+                }
+            }
+
+            last;
+        }
+    }
+
+    foreach my $video (@$results) {
+        if (exists $video->{mimeType}) {
+            $video->{type} = $video->{mimeType};
+        }
+    }
+
+    return 1;
+}
+
 sub _old_extract_streaming_urls {
     my ($self, $info, $videoID) = @_;
 
@@ -669,27 +710,7 @@ sub _old_extract_streaming_urls {
     push @results, $self->_group_keys_with_values(%stream_map);
     push @results, $self->_group_keys_with_values(%adaptive_fmts);
 
-    foreach my $video (@results) {
-        if (exists $video->{s}) {    # has an encrypted signature :(
-
-            if ($self->get_debug) {
-                say STDERR ":: Detected an encrypted signature...";
-            }
-
-            my @formats = $self->_fallback_extract_urls($videoID);
-
-            foreach my $format (@formats) {
-                foreach my $ref (@results) {
-                    if (defined($ref->{itag}) and ($ref->{itag} eq $format->{itag})) {
-                        $ref->{url} = $format->{url};
-                        last;
-                    }
-                }
-            }
-
-            last;
-        }
-    }
+    $self->_check_streaming_urls($videoID, \@results);
 
     if ($info->{livestream} or $info->{live_playback}) {
 
@@ -710,11 +731,6 @@ sub _old_extract_streaming_urls {
         }
     }
 
-    if ($self->get_debug) {
-        my $count = scalar(@results);
-        say STDERR ":: Found $count streaming URLs...";
-    }
-
     return @results;
 }
 
@@ -752,29 +768,7 @@ sub _extract_streaming_urls {
         }
     }
 
-    foreach my $item (@results) {
-
-        if (exists $item->{cipher} and not exists $item->{url}) {
-
-            my %data = $self->parse_query_string($item->{cipher});
-
-            $item->{url} = $data{url} if defined($data{url});
-
-            if (defined($data{s})) {    # unclear how this can be decrypted...
-                require URI::Escape;
-                my $sig = $data{s};
-                $sig = URI::Escape::uri_escape($sig);
-                $item->{url} .= "&sig=$sig";
-            }
-        }
-
-        if (exists $item->{mimeType}) {
-            $item->{type} = $item->{mimeType};
-        }
-    }
-
-    # Cipher streaming URLs are currently unsupported, so let's filter them out.
-    @results = grep { not exists $_->{cipher} } @results;
+    $self->_check_streaming_urls($videoID, \@results);
 
     # Keep only streams with contentLength > 0.
     @results = grep { $_->{itag} == 22 or (exists($_->{contentLength}) and $_->{contentLength} > 0) } @results;
@@ -798,11 +792,6 @@ sub _extract_streaming_urls {
         }
     }
 
-    if ($self->get_debug) {
-        my $count = scalar(@results);
-        say STDERR ":: Found $count streaming URLs...";
-    }
-
     return @results;
 }
 
