@@ -26,8 +26,14 @@ our $digit_or_equal_re   = qr/(?(?=[1-9])|=)/;
 our $non_digit_or_opt_re = qr{^(?!$range_num_re)(?>[0-9]{1,2}[^0-9]|[0-9]{3}|[^0-9$opt_begin_chars])};
 
 # Generic name
-my $generic_name_re = qr/[a-zA-Z0-9_.\-]{11,34}/;
-our $valid_channel_id_re = qr{^(?:\w+(?:[-.]++\w++)*|$generic_name_re)\z};
+my $generic_name_re = qr/[a-zA-Z0-9_.\-]{11,64}/;
+our $valid_channel_id_re = qr{^(?:.*/channel/)?(?<channel_id>(?:\w+(?:[-.]++\w++)*|$generic_name_re))(?:/.*)?\z};
+
+our $get_channel_videos_id_re    = qr{^.*/channel/(?<channel_id>(?:\w+(?:[-.]++\w++)*|$generic_name_re))};
+our $get_channel_playlists_id_re = qr{$get_channel_videos_id_re/playlists};
+
+our $get_username_videos_re    = qr{^.*/user/(?<username>[-.\w]+)};
+our $get_username_playlists_re = qr{$get_username_videos_re/playlists};
 
 # Video ID
 my $video_id_re = qr/[0-9A-Za-z_\-]{11}/;
@@ -50,6 +56,10 @@ our @EXPORT = qw(
   $valid_playlist_id_re
   $get_playlist_id_re
   $valid_opt_re
+  $get_channel_videos_id_re
+  $get_channel_playlists_id_re
+  $get_username_videos_re
+  $get_username_playlists_re
   );
 
 =head1 AUTHOR
